from os import path
from shutil import rmtree
from pyrogram import filters
from pyrogram.types import CallbackQuery, InlineKeyboardMarkup, InlineKeyboardButton
from classes.exceptions import SubmissionDuplicatesError, SubmissionUnavailableError
from classes.poster_client import PosterClient
from classes.user import PosterUser

from modules.app import app
from modules.logger import logWrite
from modules.utils import configGet, locale
from modules.database import col_submitted
from bson import ObjectId


@app.on_callback_query(filters.regex("sub_yes_[\s\S]*"))
async def callback_query_yes(app: PosterClient, clb: CallbackQuery):
    fullclb = str(clb.data).split("_")
    user_locale = clb.from_user.language_code

    db_entry = col_submitted.find_one({"_id": ObjectId(fullclb[2])})

    try:
        submission = await app.submit_photo(fullclb[2])
    except SubmissionUnavailableError:
        await clb.answer(
            text=locale("sub_msg_unavail", "callback", locale=user_locale),
            show_alert=True,
        )
        return
    except SubmissionDuplicatesError as exp:
        await clb.answer(
            text=locale("sub_duplicates_found", "callback", locale=user_locale),
            show_alert=True,
        )
        await clb.message.reply_text(
            locale("sub_media_duplicates_list", "message", locale=user_locale).format(
                "\n • ".join(exp.duplicates)
            ),
            quote=True,
        )
        logWrite(
            locale(
                "submission_duplicate",
                "console",
                locale=configGet("locale_log").format(
                    fullclb[2],
                    str(exp.duplicates),
                ),
            ),
            debug=True,
        )
        return

    if submission[0] is not None:
        await submission[0].reply_text(
            locale("sub_yes", "message", locale=submission[0].from_user.language_code),
            quote=True,
        )
    elif db_entry is not None:
        await app.send_message(db_entry["user"], locale("sub_yes", "message"))

    await clb.answer(
        text=locale("sub_yes", "callback", locale=user_locale).format(fullclb[2]),
        show_alert=True,
    )

    edited_markup = (
        [
            [
                InlineKeyboardButton(
                    text=str(locale("accepted", "button", locale=user_locale)),
                    callback_data="nothing",
                )
            ],
            clb.message.reply_markup.inline_keyboard[1],
        ]
        if len(clb.message.reply_markup.inline_keyboard) > 1
        else [
            [
                InlineKeyboardButton(
                    text=str(locale("accepted", "button", locale=user_locale)),
                    callback_data="nothing",
                )
            ]
        ]
    )

    if configGet("send_uploaded_id", "submission"):
        await clb.message.edit_caption(
            clb.message.caption + f"\n\nID: `{submission[1]}`"
        )

    await clb.message.edit_reply_markup(
        reply_markup=InlineKeyboardMarkup(edited_markup)
    )

    logWrite(
        locale(
            "submission_accepted",
            "console",
            locale=configGet("locale_log").format(fullclb[2], submission[1]),
        ),
        debug=True,
    )

    # try:
    #     if configGet("api_based", "mode") is True:
    #         media = await app.download_media(submission, file_name=configGet("tmp", "locations")+sep)
    #         upload = upload_pic(media)
    #         if upload[0] is False:
    #             await clb.answer(text=locale("sub_media_failed", "message", locale=user_locale), show_alert=True)
    #         elif len(upload[1]) > 0:
    #             await clb.answer(text=locale("sub_media_duplicates", "message", locale=user_locale))
    #             await clb.message.reply_text(locale("sub_media_duplicates_list", "message", locale=user_locale).format("\n • ".join(upload[1])))
    #         else:
    #             if clb.data.endswith("_caption"):
    #                 index = jsonLoad(configGet("index", "locations"))
    #                 index["captions"][Path(media).name] = submission.caption
    #                 jsonSave(index, configGet("index", "locations"))
    #     else:
    #         media = await app.download_media(submission, file_name=configGet("queue", "locations")+sep)
    #         if clb.data.endswith("_caption"):
    #             index = jsonLoad(configGet("index", "locations"))
    #             index["captions"][Path(media).name] = submission.caption
    #             jsonSave(index, configGet("index", "locations"))
    # except:
    #     await clb.answer(text=locale("sub_media_unavail", "message", locale=user_locale), show_alert=True)
    #     return


@app.on_callback_query(filters.regex("sub_no_[\s\S]*"))
async def callback_query_no(app: PosterClient, clb: CallbackQuery):
    fullclb = str(clb.data).split("_")
    user_locale = clb.from_user.language_code

    db_entry = col_submitted.find_one_and_delete({"_id": ObjectId(fullclb[2])})

    if db_entry["temp"]["uuid"] is not None:
        if path.exists(
            path.join(
                configGet("data", "locations"), "submissions", db_entry["temp"]["uuid"]
            )
        ):
            rmtree(
                path.join(
                    configGet("data", "locations"),
                    "submissions",
                    db_entry["temp"]["uuid"],
                ),
                ignore_errors=True,
            )

    try:
        submission = await app.get_messages(
            db_entry["user"], db_entry["telegram"]["msg_id"]
        )
    except:
        await clb.answer(
            text=locale("sub_msg_unavail", "message", locale=user_locale),
            show_alert=True,
        )
        return

    await submission.reply_text(
        locale("sub_no", "message", locale=submission.from_user.language_code),
        quote=True,
    )
    await clb.answer(
        text=locale("sub_no", "callback", locale=user_locale).format(fullclb[2]),
        show_alert=True,
    )

    edited_markup = (
        [
            [
                InlineKeyboardButton(
                    text=str(locale("declined", "button", locale=user_locale)),
                    callback_data="nothing",
                )
            ],
            clb.message.reply_markup.inline_keyboard[1],
        ]
        if len(clb.message.reply_markup.inline_keyboard) > 1
        else [
            [
                InlineKeyboardButton(
                    text=str(locale("declined", "button", locale=user_locale)),
                    callback_data="nothing",
                )
            ]
        ]
    )
    await clb.message.edit_reply_markup(
        reply_markup=InlineKeyboardMarkup(edited_markup)
    )
    logWrite(
        locale(
            "submission_rejected",
            "console",
            locale=configGet("locale_log").format(fullclb[2]),
        ),
        debug=True,
    )


@app.on_callback_query(filters.regex("sub_block_[\s\S]*"))
async def callback_query_block(app: PosterClient, clb: CallbackQuery):
    fullclb = str(clb.data).split("_")
    user_locale = clb.from_user.language_code
    await app.send_message(
        int(fullclb[2]), locale("sub_blocked", "message", locale=configGet("locale"))
    )
    PosterUser(int(fullclb[2])).block()
    await clb.answer(
        text=locale("sub_block", "callback", locale=user_locale).format(fullclb[2]),
        show_alert=True,
    )

    edited_markup = [
        clb.message.reply_markup.inline_keyboard[0],
        [
            InlineKeyboardButton(
                text=str(locale("sub_unblock", "button", locale=user_locale)),
                callback_data=f"sub_unblock_{fullclb[2]}",
            )
        ],
    ]
    await clb.message.edit_reply_markup(
        reply_markup=InlineKeyboardMarkup(edited_markup)
    )
    logWrite(
        locale(
            "user_blocked",
            "console",
            locale=configGet("locale_log").format(fullclb[2]),
        ),
        debug=True,
    )


@app.on_callback_query(filters.regex("sub_unblock_[\s\S]*"))
async def callback_query_unblock(app: PosterClient, clb: CallbackQuery):
    fullclb = str(clb.data).split("_")
    user_locale = clb.from_user.language_code
    await app.send_message(
        int(fullclb[2]), locale("sub_unblocked", "message", locale=configGet("locale"))
    )
    PosterUser(int(fullclb[2])).unblock()
    await clb.answer(
        text=locale("sub_unblock", "callback", locale=user_locale).format(fullclb[2]),
        show_alert=True,
    )

    edited_markup = [
        clb.message.reply_markup.inline_keyboard[0],
        [
            InlineKeyboardButton(
                text=str(locale("sub_block", "button", locale=user_locale)),
                callback_data=f"sub_block_{fullclb[2]}",
            )
        ],
    ]
    await clb.message.edit_reply_markup(
        reply_markup=InlineKeyboardMarkup(edited_markup)
    )
    logWrite(
        locale(
            "user_unblocked",
            "console",
            locale=configGet("locale_log").format(fullclb[2]),
        ),
        debug=True,
    )
